expect = require('chai').expect

OP_GET_GIST = '99d64f8e-eace-417d-8cea-511621aaf57c'
GIST_ID = '6a06816491046e57e658'
GIST_RESOURCE = '5cb3434f-0351-461b-9fe0-89d11b2c046c'
OP_GTABLE_GETTABLE = '7810abb5-1617-452a-a83d-9f0349da460c'
OP_GTABLE_SQLGET = 'aa746210-294a-4e57-bff0-4ba05d948645'
TABLE_ID = '1EKWSEI4JtA8xYWou_u_BKmEHjrmXHTguNbct_TI'
GTABLE_API_KEY = process.env.GTABLE_API_KEY
OP_SOCRATA_GETRESOURCE='c1b1bf41-a4e6-4562-a880-72d9165a604e'
SOCRATA_APP_TOKEN = process.env.SOCRATA_APP_TOKEN
SC_RESOURCE_ID = 'e3fb-j9c6'

describe 'APIBase library', ->
  it 'should return an api client', ->
    APIBase = require '../src'
    client = new APIBase({
      baseUrl: 'http://johnsd.cse.unsw.edu.au:3000'
    })
    expect(client).to.exist

describe 'APIBase client', ->
  APIBase = require '../src'
  client = new APIBase({
    baseUrl: 'http://johnsd.cse.unsw.edu.au:3000'
  })
  
  it 'should get an entity', (done)->
    client.getEntity OP_GET_GIST, (err, entity)->
      expect(entity.name).to.equal("GetGist")
      expect(entity.id).to.equal(OP_GET_GIST)
      done()

  it 'should return a Gist', (done)->
    @timeout(20000)
    client.invokeOperation OP_GET_GIST, {gistId: GIST_ID}, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'url', "https://api.github.com/gists/#{GIST_ID}"
      expect(resp).to.have.property 'files'
      expect(resp.files).to.have.property 'route.c'
      done(err)

  it 'should return a Gist (resource mapping)', (done)->
    @timeout(20000)
    client.invokeOperation OP_GET_GIST, {}, {gistIda: GIST_ID}, {"gistId": "$.gistIda"}, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'url', "https://api.github.com/gists/#{GIST_ID}"
      expect(resp).to.have.property 'files'
      expect(resp.files).to.have.property 'route.c'
      done(err)

  it 'should return a Gist (resource)', (done)->
    @timeout(30000)
    client.invokeOperation OP_GET_GIST, {gistId: "resource://#{GIST_RESOURCE}"}, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'url', "https://api.github.com/gists/6af050afb253caa2cd67"
      expect(resp).to.have.property 'files'
      expect(resp.files).to.have.property 'HelloWorld.js'
      done(err)

describe 'Google Table', ->
  nock = require 'nock'
  ft = nock('https://www.googleapis.com')
  ft.get '/fusiontables/v2/query'
    .query true
    .reply 200, {
      "kind": "fusiontables#sqlresponse"
      "columns": [
        "Browser"
        "Minimum Version"
        "Maximum Version"
        "Export to Excel/Word/Mail Merge"
        "QuickBooks Extension"
      ]
    }
  ft.get("/fusiontables/v2/tables/#{TABLE_ID}")
    .query(true)
    .reply 200, {
      "kind": "fusiontables#table"
      "name": "Configuring Your Browser Settings"
    }
  APIBase = require '../src'
  client = new APIBase({
    baseUrl: 'http://johnsd.cse.unsw.edu.au:3000'
  })

  it 'should return a Google Table Metadata', (done)->
    @timeout(20000)
    client.invokeOperation OP_GTABLE_GETTABLE, {tableId: TABLE_ID, key: GTABLE_API_KEY}, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'kind', 'fusiontables#table'
      expect(resp).to.have.property 'name', 'Configuring Your Browser Settings'
      done(err)

  it 'should return a Google Table', (done)->
    @timeout(20000)
    client.invokeOperation OP_GTABLE_SQLGET, {
      key: GTABLE_API_KEY,
      sql: "SELECT * FROM #{TABLE_ID}"
    }, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'kind', 'fusiontables#sqlresponse'
      expect(resp).to.have.property('columns')
        .that.is.an('array')
        .with.length(5)
        .and.contains('Browser')
      done(err)

describe 'Socrata API', ->
  client = null
  beforeEach ->
    APIBase = require '../src'
    client = new APIBase({
      baseUrl: 'http://johnsd.cse.unsw.edu.au:3000'
    })

  it 'should return a Socrata resource', (done)->
    @timeout(20000)
    client.invokeOperation OP_SOCRATA_GETRESOURCE, {id: SC_RESOURCE_ID, 'X-App-Token': SOCRATA_APP_TOKEN}, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.length(5)
      expect(resp[0]).to.have.property 'pcworld', '1.36'
      done(err)

  it 'should return a Socrata resource', (done)->
    @timeout(20000)
    client.invokeOperation OP_SOCRATA_GETRESOURCE, {
      id: SC_RESOURCE_ID,
      'X-App-Token': SOCRATA_APP_TOKEN,
      '$select': 'yahoo,ebay'
      '$where': 'yahoo>1.3'
    }, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.length(3)
      expect(resp[0]).to.have.property 'ebay', '1.13'
      done(err)

  it 'should return a failure message', (done)->
    @timeout(20000)
    client.invokeOperation OP_SOCRATA_GETRESOURCE, {id: SC_RESOURCE_ID, 'X-App-Token': SOCRATA_APP_TOKEN + 'x'}, null, null, (err, resp)->
      expect(resp).to.exist
      expect(resp).to.have.property 'error'
      done(err)
    
describe 'caching', ->
  APIBase = require '../src'
  client = new APIBase({
    baseUrl: 'http://johnsd.cse.unsw.edu.au:3000'
  })

  it 'should get an entity', (done)->
    client.getEntity OP_GET_GIST, (err, entity)->
      expect(entity.name).to.equal("GetGist")
      expect(entity.id).to.equal(OP_GET_GIST)
      done()

  it 'should get an entity immediately', (done)->
    @timeout(20)
    client.getEntity OP_GET_GIST, (err, entity)->
      expect(entity.name).to.equal("GetGist")
      expect(entity.id).to.equal(OP_GET_GIST)
      done()

  it 'should get api of an operation', (done)->
    @timeout(6000)
    client._getOperationAPI OP_GET_GIST, (err, api)->
      expect(api.name).to.equal("Github")
      done()

  it 'should get api of an operation immediately', (done)->
    @timeout(20)
    client._getOperationAPI OP_GET_GIST, (err, api)->
      expect(api.name).to.equal("Github")
      done()
